"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var nopt_1 = __importDefault(require("nopt"));
var path_1 = __importDefault(require("path"));
var fs_1 = __importDefault(require("fs"));
var chokidar_1 = __importDefault(require("chokidar"));
var async_1 = __importDefault(require("async"));
var logger_1 = __importDefault(require("./lib/logger"));
var util_1 = __importDefault(require("./lib/util"));
var eustia = require('./index');
var knowOpts = {
    encoding: String,
    extension: Array,
    output: String,
    update: Boolean,
    namespace: String,
    ignore: Array,
    keyword: String,
    library: Array,
    exclude: Array,
    include: Array,
    format: String,
    title: String,
    config: String,
    watch: Boolean,
    verbose: Boolean,
    help: Boolean,
    strict: Boolean,
    version: Boolean,
    description: String,
    proxy: String,
    ts: Boolean,
    sourcemap: Boolean
};
var shortHands = {
    o: '--output',
    i: '--include',
    k: '--keyword',
    l: '--library',
    e: '--exclude',
    n: '--namespace',
    f: '--format',
    s: '--strict',
    t: '--title',
    w: '--watch',
    u: '--update',
    c: '--config',
    h: '--help',
    v: '--version',
    d: '--description'
};
var options = nopt_1["default"](knowOpts, shortHands, process.argv, 2);
var remain = options.argv.remain;
logger_1["default"].enable();
options.enableLog = true;
options.errLog = true;
var cmd = getCmd();
cmd ? useCmdLine() : useCfg();
function getCmd() {
    var i, len, ret;
    for (i = 0, len = remain.length; i < len; i++) {
        if (util_1["default"].has(eustia, remain[i])) {
            ret = remain[i];
            remain.splice(i, 1);
            break;
        }
    }
    if (!ret) {
        if (options.help) {
            ret = 'help';
        }
        else if (options.version) {
            ret = 'version';
        }
    }
    return ret;
}
function useCfg() {
    var configs;
    var cfgPaths = [
        options.config || '.eustia',
        '.eustia.js',
        '.eustia.json',
        '.eustiarc',
        '.eustiarc.js',
        '.eustiarc.json'
    ].map(function (val) { return path_1["default"].resolve(process.cwd(), val); });
    async_1["default"].detect(cfgPaths, function (filePath, cb) {
        fs_1["default"].access(filePath, function (err) {
            cb(null, !err);
        });
    }, function (err, cfgPath) {
        if (cfgPath) {
            logger_1["default"].tpl({ data: cfgPath }, 'CONFIGURATION FILE {{#cyan}}{{{data}}}{{/cyan}}');
            return fs_1["default"].readFile(cfgPath, 'utf8', function (err, data) {
                try {
                    configs = JSON.parse(data);
                }
                catch (e) {
                    configs = require(cfgPath);
                }
                build(configs);
            });
        }
        try {
            var pkgInfo = require(path_1["default"].resolve(process.cwd(), 'package.json'));
            if (pkgInfo.eustia) {
                logger_1["default"].tpl({ data: 'package.json' }, 'CONFIGURATION FILE {{#cyan}}{{{data}}}{{/cyan}}');
                build(pkgInfo.eustia);
            }
            else {
                eustia.help(options);
            }
        }
        catch (e) {
            eustia.help(options);
        }
    });
}
function useCmdLine() {
    var hasRemain = !util_1["default"].isEmpty(remain);
    switch (cmd) {
        case 'build':
            options.files = remain;
            break;
        case 'search':
            if (hasRemain)
                options.keyword = remain[0];
            break;
        case 'help':
            if (hasRemain)
                options.command = remain[0];
            break;
        case 'doc':
            if (hasRemain)
                options.input = remain[0];
            break;
        case 'install':
            options.modules = remain;
            break;
        case 'cache':
            if (hasRemain)
                options.subCmd = remain[0];
            break;
    }
    eustia[cmd](options);
}
function build(configs) {
    var isSingle = util_1["default"].some(configs, function (option) {
        return !util_1["default"].isPlainObj(option);
    });
    if (isSingle) {
        util_1["default"].extend(configs, options);
        return eustia.build(configs);
    }
    buildAll(configs);
}
function buildAll(configs) {
    var isTaskSpecified = remain.length > 0 && configs[remain[0]];
    if (isTaskSpecified) {
        logger_1["default"].tpl({}, 'Run task {{#cyan}}"' + remain[0] + '"{{/cyan}}:');
        return eustia['build'](util_1["default"].extend(configs[remain[0]], options));
    }
    var cfgArr = [];
    var watch = options.watch;
    options.watch = false;
    util_1["default"].each(configs, function (val, key) {
        val.taskName = key;
        cfgArr.push(val);
        configs[key] = util_1["default"].extend(val, options);
    });
    var cfgLen = cfgArr.length;
    var i = 0;
    function build(config, isWatching) {
        logger_1["default"].tpl({}, 'Run task {{#cyan}}"' + config.taskName + '"{{/cyan}}:');
        eustia.build(config, function () {
            if (++i < cfgLen)
                return build(cfgArr[i], isWatching);
            if (!isWatching && watch)
                beginWatch();
        });
    }
    build(cfgArr[i]);
    function beginWatch() {
        var watchPaths = [];
        var outputs = [];
        util_1["default"].each(cfgArr, function (config) {
            outputs = outputs.concat(config.output);
            watchPaths = watchPaths.concat(config.files);
            watchPaths = watchPaths.concat(config.libPaths);
        });
        chokidar_1["default"]
            .watch(watchPaths, {
            persistent: true,
            ignored: outputs,
            ignoreInitial: true,
            followSymlinks: true,
            usePolling: true,
            alwaysStat: false,
            interval: 100,
            atomic: true,
            ignorePermissionErrors: false
        })
            .on('change', function () {
            i = 0;
            build(cfgArr[i], true);
        });
    }
}
