"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var path_1 = __importDefault(require("path"));
var logger_1 = __importDefault(require("../../lib/logger"));
var readPaths_1 = __importDefault(require("../../lib/readPaths"));
var util_1 = __importDefault(require("../../lib/util"));
function default_1(options, cb) {
    if (util_1["default"].isEmpty(options.files)) {
        options.data.fnPercentage = {};
        return cb(null, options.include.sort());
    }
    logger_1["default"].log('SCAN FILES');
    logger_1["default"].color(options.files.join('\n'), 'cyan');
    var files = options.files;
    var modList = options.include;
    readPaths_1["default"](files, options, function (err, files) {
        if (err) {
            return cb(err);
        }
        files.forEach(function (file) {
            if (util_1["default"].startWith(file.data, options.magicNum)) {
                return;
            }
            file.data = util_1["default"].stripCmt(file.data);
            var modules = extractModule(file, options);
            if (modules.length !== 0) {
                logger_1["default"].debug('File', file.path, 'has', modules);
            }
            modList = modList.concat(modules);
        });
        modList = modList.map(function (fnName) {
            return util_1["default"].trim(fnName);
        });
        options.data.fnPercentage = getFnPercentage(modList, files.length);
        modList = util_1["default"].unique(modList).filter(function (fnName) {
            return !util_1["default"].contain(options.exclude, fnName);
        });
        cb(null, modList.sort());
    });
}
exports["default"] = default_1;
var regCommonjs = /require\(.*\)/;
var regEs6 = /\bimport\b/;
function extractModule(file, options) {
    var ret = [];
    if (regCommonjs.test(file.data)) {
        ret = ret.concat(extractCommonjs(options, file));
    }
    if (regEs6.test(file.data)) {
        ret = ret.concat(extractEs6(options, file));
    }
    ret = ret.concat(extractGlobal(options.namespace, file));
    ret = util_1["default"].map(ret, function (module) { return util_1["default"].trim(module); });
    ret = util_1["default"].filter(ret, function (module) { return module !== ''; });
    return util_1["default"].unique(ret);
}
var regModules = {
    _cache: {},
    get: function (namespace) {
        var ret;
        ret = this._cache[namespace];
        if (ret) {
            return ret;
        }
        ret = this._cache[namespace] = new RegExp('\\b' + namespace + '((\\.[\\$_\\w]+)|(\\[[\'"][\\$_\\w]+[\'"]\\]))', 'g');
        return ret;
    }
};
function extractGlobal(namespace, file) {
    var modules = file.data.match(regModules.get(namespace));
    return modules
        ? util_1["default"].map(modules, function (val) {
            val = val.substr(namespace.length);
            val = val[0] === '[' ? val.slice(2, -2) : val.slice(1);
            return val;
        })
        : [];
}
function extractCommonjs(options, file) {
    var requirePath = relativePath(file.path, options.output);
    var regRequire = new RegExp('(\\w+)\\s*=\\s*require\\([\'"]' + requirePath + '(?:\\.js)?[\'"]\\)');
    var namespace = file.data.match(regRequire);
    if (namespace) {
        namespace = namespace[1];
    }
    return extractGlobal(namespace ? namespace : options.namespace, file);
}
function relativePath(from, to) {
    var ret = path_1["default"]
        .relative(path_1["default"].dirname(from), to)
        .replace(/\\/g, '/')
        .replace(/\.js$/, '')
        .replace(/\./g, '\\.');
    if (!util_1["default"].startWith(ret, '\\.')) {
        ret = '\\./' + ret;
    }
    return ret;
}
function extractEs6(options, file) {
    var ret = [];
    var requirePath = relativePath(file.path, options.output);
    var regImport = new RegExp('import\\s+(\\w+)\\s+from\\s*[\'"]' + requirePath + '(?:\\.js)?[\'"]');
    var namespace = file.data.match(regImport);
    if (namespace) {
        namespace = namespace[1];
        ret = ret.concat(extractGlobal(namespace, file));
    }
    requirePath = relativePath(file.path, options.output);
    regImport = new RegExp('import\\s+(\\*\\s+as)\\s+(\\w+)\\s+from\\s*[\'"]' +
        requirePath +
        '(?:\\.js)?[\'"]');
    namespace = file.data.match(regImport);
    if (namespace) {
        namespace = namespace[2];
        ret = ret.concat(extractGlobal(namespace, file));
    }
    var regImportMembers = new RegExp('import\\s*{([\\w,\\$\\s]+)}\\s*from\\s*[\'"]' +
        requirePath +
        '(?:\\.js)?[\'"]');
    var methods = file.data.match(regImportMembers);
    if (methods) {
        ret = ret.concat(methods[1].split(','));
    }
    return ret;
}
function getFnPercentage(fnList, filesNum) {
    var ret = {};
    fnList.forEach(function (fnName) {
        ret[fnName] = ret[fnName] !== undefined ? ret[fnName] + 1 : 1;
    });
    util_1["default"].each(ret, function (val, key) {
        ret[key] = ((val / filesNum) * 100).toFixed(2) + '%';
    });
    return ret;
}
