"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var build_1 = __importDefault(require("./cmd/build"));
var doc_1 = __importDefault(require("./cmd/doc"));
var help_1 = __importDefault(require("./cmd/help"));
var version_1 = __importDefault(require("./cmd/version"));
var logger_1 = __importDefault(require("./lib/logger"));
var util_1 = __importDefault(require("./lib/util"));
var cwd = process.cwd();
var defOpts = {
    cwd: cwd,
    dirname: path_1["default"].resolve(__dirname, '../'),
    magicNum: '// Built by eustia.',
    data: {},
    enableLog: false,
    debug: false,
    encoding: 'utf8',
    errLog: false,
    packInfo: require('../package.json')
};
var errLogPath = path_1["default"].resolve(cwd, './eustia-debug.log');
module.exports = {
    build: cmdFactory(build_1["default"]),
    doc: cmdFactory(doc_1["default"]),
    help: cmdFactory(help_1["default"]),
    version: cmdFactory(version_1["default"])
};
function cmdFactory(cmd) {
    return function (options, cb) {
        cb = cb || util_1["default"].noop;
        options = util_1["default"].defaults(options, defOpts, cmd.defOpts || {});
        options.cacheDir = path_1["default"].resolve(options.cwd, 'eustia/cache');
        if (options.enableLog) {
            logger_1["default"].enable();
        }
        if (options.verbose) {
            logger_1["default"].isDebug = true;
        }
        logger_1["default"].debug('Options', options);
        cmd(options, function (err, result) {
            if (err) {
                logger_1["default"].error(err);
                cb(err);
                if (options.errLog) {
                    fs_1["default"].writeFileSync(errLogPath, logger_1["default"].history(), 'utf-8');
                    process.exit();
                }
            }
            if (options.errLog) {
                fs_1["default"].exists(errLogPath, function (result) {
                    if (result) {
                        fs_1["default"].unlink(errLogPath, util_1["default"].noop);
                    }
                });
            }
            cb(null, result);
        });
    };
}
