"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var handlebars_1 = __importDefault(require("handlebars"));
var util_1 = require("util");
var util_2 = __importDefault(require("./util"));
var Logger = (function () {
    function Logger() {
        this.isDebug = false;
        this.isEnabled = false;
        this.msgs = [];
        this.initHandlebars();
    }
    Logger.prototype.info = function (msg) {
        this.color(msg, 'green');
    };
    Logger.prototype.error = function (err) {
        this.color(err.message, 'red', err);
    };
    Logger.prototype.log = function (msg) {
        this.color(msg, 'white');
    };
    Logger.prototype.debug = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (!this.isDebug) {
            return;
        }
        this.tpl({ data: util_1.format.apply(null, args) }, '{{#red}}DEBUG{{/red}} {{{data}}}');
    };
    Logger.prototype.warn = function (msg) {
        this.color(msg, 'yellow');
    };
    Logger.prototype.color = function (msg, color, err) {
        this.tpl({ msg: msg }, '{{#' + color + '}}{{{msg}}}{{/' + color + '}}', err);
    };
    Logger.prototype.history = function () {
        return util_2["default"].stripColor(this.msgs.join('\n'));
    };
    Logger.prototype.enable = function () {
        this.isEnabled = true;
    };
    Logger.prototype.tpl = function (msg, tpl, err) {
        msg = handlebars_1["default"].compile(tpl)(msg);
        this._log(msg, err);
    };
    Logger.prototype._log = function (msg, err) {
        if (!this.isEnabled) {
            return;
        }
        this.msgs.push(err ? err.stack : msg);
        process.stdout.write(msg + '\n');
    };
    Logger.prototype.initHandlebars = function () {
        handlebars_1["default"].registerHelper('rapd', function (len, ctx) {
            return util_2["default"].rpad(ctx.fn(this), +len, ' ');
        });
        var SUPPORT_COLORS = [
            'yellow',
            'green',
            'cyan',
            'red',
            'white',
            'magenta'
        ];
        SUPPORT_COLORS.forEach(function (color) {
            handlebars_1["default"].registerHelper(color, function (ctx) {
                return util_2["default"].ansiColor[color](ctx.fn(this));
            });
        });
    };
    return Logger;
}());
exports["default"] = new Logger();
