"use strict";
var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var async_1 = __importDefault(require("async"));
var fs_1 = __importDefault(require("fs"));
var handlebars_1 = __importDefault(require("handlebars"));
var path_1 = __importDefault(require("path"));
var logger_1 = __importDefault(require("./logger"));
var util_1 = __importDefault(require("./util"));
var tpl = {};
var template = {
    amd: util_1["default"].stripIndent(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n    define([], function () {\n      {{codes}}\n    });\n  "], ["\n    define([], function () {\n      {{codes}}\n    });\n  "]))),
    code: util_1["default"].stripIndent(templateObject_2 || (templateObject_2 = __makeTemplateObject(["\n    /* ------------------------------ {{name}} ------------------------------ */\n\n    {{#if hasExports}}\n    {{#if es}}export {{/if}}var {{name}} = _.{{name}} = (function (exports) {\n        {{code}}\n    \n        return exports;\n    })({});\n    {{else}}\n    (function () {\n        {{code}}\n    })();\n    {{/if}}\n  "], ["\n    /* ------------------------------ {{name}} ------------------------------ */\n\n    {{#if hasExports}}\n    {{#if es}}export {{/if}}var {{name}} = _.{{name}} = (function (exports) {\n        {{code}}\n    \n        return exports;\n    })({});\n    {{else}}\n    (function () {\n        {{code}}\n    })();\n    {{/if}}\n  "]))),
    codes: util_1["default"].stripIndent(templateObject_3 || (templateObject_3 = __makeTemplateObject(["\n    /* eslint-disable */\n    {{#if strict}}\"use strict\";{{/if}}\n    \n    var _ = {};\n    \n    {{#if inBrowser}}\n    if (typeof window === 'object' && window.{{namespace}}) _ = window.{{namespace}};\n    \n    {{#if excludeRef}}\n    {{#each excludeRef}}{{#if @first}}var {{else}}    {{/if}}{{.}} = _.{{.}}{{#if @last}};{{else}},{{/if}}\n    {{/each}}\n    {{/if}}\n    {{/if}}\n    \n    {{code}}\n    \n    {{#if commonjs}}module.exports = _;{{else if es}}export default _;{{else}}return _;{{/if}}\n  "], ["\n    /* eslint-disable */\n    {{#if strict}}\"use strict\";{{/if}}\n    \n    var _ = {};\n    \n    {{#if inBrowser}}\n    if (typeof window === 'object' && window.{{namespace}}) _ = window.{{namespace}};\n    \n    {{#if excludeRef}}\n    {{#each excludeRef}}{{#if @first}}var {{else}}    {{/if}}{{.}} = _.{{.}}{{#if @last}};{{else}},{{/if}}\n    {{/each}}\n    {{/if}}\n    {{/if}}\n    \n    {{code}}\n    \n    {{#if commonjs}}module.exports = _;{{else if es}}export default _;{{else}}return _;{{/if}}\n  "]))),
    global: util_1["default"].stripIndent(templateObject_4 || (templateObject_4 = __makeTemplateObject(["\n    window.{{namespace}} = (function()\n    {\n    {{codes}}\n    })();\n  "], ["\n    window.{{namespace}} = (function()\n    {\n    {{codes}}\n    })();\n  "]))),
    umd: util_1["default"].stripIndent(templateObject_5 || (templateObject_5 = __makeTemplateObject(["\n    (function(root, factory)\n    {\n        if (typeof define === 'function' && define.amd)\n        {\n            define([], factory);\n        } else if (typeof module === 'object' && module.exports)\n        {\n            module.exports = factory();\n        } else { root.{{namespace}} = factory(); }\n    }(this, function ()\n    {\n    {{codes}}\n    }));\n  "], ["\n    (function(root, factory)\n    {\n        if (typeof define === 'function' && define.amd)\n        {\n            define([], factory);\n        } else if (typeof module === 'object' && module.exports)\n        {\n            module.exports = factory();\n        } else { root.{{namespace}} = factory(); }\n    }(this, function ()\n    {\n    {{codes}}\n    }));\n  "])))
};
function readTpl(tplName) {
    return function (cb) {
        if (template[tplName]) {
            tpl[tplName] = handlebars_1["default"].compile(template[tplName], { noEscape: true });
            cb();
        }
        else {
            var tplPath = path_1["default"].resolve(__dirname, '../../tpl/' + tplName + '.hbs');
            logger_1["default"].debug('Read tpl', tplPath);
            fs_1["default"].readFile(tplPath, 'utf8', function (err, data) {
                if (err) {
                    return cb(err);
                }
                tpl[tplName] = handlebars_1["default"].compile(data, { noEscape: true });
                cb();
            });
        }
    };
}
function default_1(templates, cb) {
    var callbacks = templates.map(function (val) {
        return readTpl(val);
    });
    async_1["default"].parallel(callbacks, function (err) {
        if (err) {
            return cb(err);
        }
        cb(null, tpl);
    });
}
exports["default"] = default_1;
var templateObject_1, templateObject_2, templateObject_3, templateObject_4, templateObject_5;
